-- main for mask.love

function love.load()
	gr, li, pi				= love.graphics, love.image, math.pi
	gr.setCaption( 'Mask' )
	smallfont, bigfont		= gr.newFont(18), gr.newFont(120)
	image					= gr.newImage( 'test.jpg')
	mask = {
		size				=  50,	-- hole size
		sides				=   5,	-- # sides hole has
		angle				=   0,	-- orbit of rotating mask
		radx				= 250,
		rady				= 250,
		width				= image:getWidth(),	-- canvas size
		height				= image:getHeight(),-- can be smaller
		foreground_alpha	= 255,
		background_alpha	=  30
		}
	mask.canvas				= gr.newCanvas( mask.width, mask.height )
	load_effect()
	load_mask()
	info = {
			" Keyboard Controls",
			"<q><w> mask size",
			"<a><s>  # sides",
			"<z><x> masks alpha",
			"<j><k>  background alpha"
			}
	end

function love.draw()
	gr.setPixelEffect( effect )
	gr.draw( image )
	gr.setPixelEffect()
	gr.print( 'FPS: '..love.timer.getFPS(), gr.getWidth()-100, 10 )
	key_options()
	show_data()
	end

function love.update( dt )
	mask.angle		=  mask.angle + dt
	if mask.angle	>= 2*pi then mask.angle = mask.angle - 2*pi end
	load_mask()
	end

function love.keypressed(key)
	if key == 'escape'	then love.event.push('quit') end
	if key == 'q'		then mask.size				= mask.size - 10														end
	if key == 'w'		then mask.size				= mask.size + 10														end
	if key == 'j'		then mask.background_alpha	= mask.background_alpha >= 25	and mask.background_alpha - 25	or 0	end
	if key == 'k'		then mask.background_alpha	= mask.background_alpha	<= 220	and mask.background_alpha + 25	or 255	end
	if key == 'z' 		then mask.foreground_alpha	= mask.foreground_alpha >=  25	and mask.foreground_alpha - 25	or 0	end
	if key == 'x'		then mask.foreground_alpha	= mask.foreground_alpha <  220	and mask.foreground_alpha + 25	or 255	end
	if key == 's'		then mask.sides 			= mask.sides < 20 				and mask.sides + 1 				or 20	end
	if key == 'a'		then mask.sides 			= mask.sides >  3 				and mask.sides - 1				or 3	end
	end

function mask_content()
	mask.canvas:clear( 0,0,0,mask.background_alpha )		-- black background
	gr.setColor( 255, 255, 255, mask.foreground_alpha )		-- white hole in mask
	local x 			= mask.radx * math.sin( -mask.angle ) + mask.width / 2
	local y 			= mask.rady * math.cos( -mask.angle ) + mask.height / 2
	local shape			= figure( mask.sides, mask.size, { x, y } )
	gr.polygon( 'fill', unpack( shape ) )
	gr.setFont( bigfont )
	msg = "LOVE"
	gr.printf( msg, x-160, y-70, 500, 'left' )
	gr.setFont( smallfont )
	end

function load_effect()
	effect = gr.newPixelEffect [[
	extern Image  mask;
	vec4 effect(vec4 color,Image tex,vec2 tc,vec2 pc)
	{
		vec4 img_color		= Texel( tex, tc );
		tc.y				= 1 - tc.y;	// invert 'y' axes
		vec4 mask_color		= Texel( mask, tc );
		img_color.a 		= mask_color.a;
		return img_color;
		}
	]]
	end

function load_mask()
	mask.canvas:renderTo( mask_content )
	effect:send( 'mask', mask.canvas )
	end

function key_options()
	gr.setColor( 155, 155, 155, 255 )
	for i = 1, #info do gr.print( info[i], 10, i * 20 ) end
	end

function show_data()
	data = {
		"Settings",
		"mask sides: "..mask.sides,
		"mask size: "..mask.size,
		"mask alpha: "..mask.foreground_alpha,
		"backgnd alpha: "..mask.background_alpha,
		"rotation angle: "..math.floor(mask.angle*180/pi)
		}
	local x, y = 10, 20
	for i = 1, #data do
		gr.print( data[i], x, i * y+ 440 )
		end
	end

function figure( number_of_sides,size,loc )	-- 2D figure with specified # of sides
	local angle_step	= 360 / number_of_sides
	local cords, index	= {}, 1		-- cords is a tbale {x1,y1,x2,y2 ... } for polygon
	for side			= 1, number_of_sides do
		cords[index]	= size*math.sin( (angle_step*(side-1)) * pi / 180 - mask.angle ) / 2+loc[1]
		index			= index + 1
		cords[index]	= size*math.cos( (angle_step*(side-1)) * pi / 180 + mask.angle ) / 2+loc[2]
		index			= index + 1
		end
	return cords
	end
